#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=C0301,C0116,W0613,W1514,C0103,C0103
import json
import os
import re
from os.path import exists
import socket
from time import sleep
from urllib.parse import urlencode
from urllib.error import HTTPError

from setting.globalconfig import GlobalConfig
from src import logger
from src.pms._base import _Base
from src.pms._base import runs_id_cmd_log


class Send2Pms(_Base):
    """发送数据到pms"""

    __author__ = "huangmingqiang@uniontech.com"

    def post_to_pms(self, run_case_id=None, case_id=None, result=None, **kwargs):
        base_url = "https://pms.uniontech.com/testtask-runCase"
        # 每次请求时，获取steps_id
        run_case_html_url = f"{base_url}-{run_case_id}.html"
        res = self.rx.open_url(run_case_html_url, timeout=10)
        if res == "":
            # 测试套件回填时地址不一样
            run_case_html_url = f"{base_url}-0-{run_case_id}-1.html"
            res = self.rx.open_url(run_case_html_url, timeout=30)
        steps_id = re.findall(r"name='steps\[(\d+)\]'", res)
        if steps_id and steps_id[0]:
            steps_id = steps_id[0]
        else:
            return 201
        # 构造post数据
        data = {
            f"steps[{steps_id}]": result,
            f"reals[{steps_id}]": "",
            "case": case_id,
            "version": "1",
            f"labels{steps_id}[]": "",
            f"files{steps_id}[]": "",
        }
        bytes_data = urlencode(data).encode("utf-8")
        # post请求接口
        res = self.rx.session.open(fullurl=run_case_html_url, data=bytes_data, timeout=30)
        status_code = res.status
        return status_code

    def send2pms(self, case_res_path, data_send_result_csv):
        """发送数据到PMS"""
        if exists(case_res_path):
            for case_name_json in os.listdir(case_res_path):
                if not case_name_json.endswith(".json"):
                    continue
                with open(f"{case_res_path}/{case_name_json}", "r", encoding="utf-8") as f:
                    data = json.load(f)
                if not exists(data_send_result_csv):
                    with open(data_send_result_csv, "w+", encoding="utf-8") as f:
                        pass
                with open(data_send_result_csv, "r", encoding="utf-8") as f:
                    reqeusted = f.read()
                case_name = case_name_json.split(".")[0]
                case_result = data.get("result")
                if case_name not in reqeusted:
                    self.push(data_send_result_csv, data, case_name, case_result)
                else:
                    if f"{case_name},fail" in reqeusted and case_result == "cover-pass":
                        fix_requested = re.sub(rf"{case_name},fail,request_.*?\n", "", reqeusted)
                        with open(data_send_result_csv, "w+", encoding="utf-8") as f:
                            f.write(fix_requested)
                        self.push(data_send_result_csv, data, case_name, case_result)
                    elif f"{case_name},pass" in reqeusted and case_result == "fail":
                        fix_requested = re.sub(rf"{case_name},pass,request_.*?\n", "", reqeusted)
                        with open(data_send_result_csv, "w+", encoding="utf-8") as f:
                            f.write(fix_requested)
                        self.push(data_send_result_csv, data, case_name, case_result)

    def push(self, data_send_result_csv, data, case_name, case_result):
        if data["result"] == "cover-pass":
            data["result"] = "pass"
        data.pop("item")
        with open(data_send_result_csv, "a+", encoding="utf-8") as f:
            for _ in range(int(GlobalConfig.SEND_PMS_RETRY_NUMBER)):
                try:
                    status_code = self.post_to_pms(**data)
                    if status_code == 200:
                        logger.info(f"{runs_id_cmd_log(data)} 数据回填成功 ✔")
                        if case_result == "cover-pass" and f"{case_name},pass" in f.read():
                            break
                        f.write(f"{case_name},{data['result']},request_ok\n")
                        break
                except (HTTPError, socket.timeout):
                    sleep(1)
            else:
                logger.info(f"{runs_id_cmd_log(data)} 数据回填失败 ✘")
                f.write(f"{case_name},{data['result']},request_fail\n")

    @staticmethod
    def case_res_path(taskid):
        """case_res_path"""
        return f"{GlobalConfig.REPORT_PATH}/pms_{taskid}"

    @classmethod
    def data_send_result_csv(cls, taskid):
        """data_send_result_csv"""
        return f"{cls.case_res_path(taskid)}/send_pms_{taskid}.csv"

    def remote_finish_push(self, res):
        """remote_finish_push"""
        for data in res.values():
            data.pop("item")
            if data["result"] == "cover-pass":
                data["result"] = "pass"
            for _ in range(int(GlobalConfig.SEND_PMS_RETRY_NUMBER)):
                try:
                    status_code = self.post_to_pms(**data)
                    if status_code == 200:
                        logger.info(f"{runs_id_cmd_log(data)} 数据回填成功 ✔")
                        break
                except (HTTPError, socket.timeout):
                    sleep(1)

            else:
                logger.info(f"{runs_id_cmd_log(data)} 数据回填失败 ✘")



# -*- coding: UTF-8 -*-
# This file was modified by UnionTech Software Technology Co., Ltd. in 2022/08/11
"""
# ==============================================
# huangmingqiang@uniontech.com
# Time: 2022/8/11
# 官方仓库 https://gitlab.com/dogtail/dogtail/
# 里面有0.9.11的版本，经过流水线测试，偶现一些奇怪的报错，
# 在pypi里面只发布了0.9.10，因此我们仍然使用0.9.10稳定版本，
# 在源代码基础上，我们新增了Wayland下的键操作支持，以兼容
# Wayland模式下的自动化用例执行。
# ==============================================
"""
from __future__ import absolute_import, division, print_function, unicode_literals
# pylint: disable=W0105,C0413,C0114
"""
GUI test tool and automation framework that uses Accessibility (a11y) technologies to communicate
with desktop applications.
"""

__author__ = """Zack Cerza <zcerza@redhat.com>,
Ed Rousseau <rousseau@redhat.com>,
David Malcolm <dmalcolm@redhat.com>,
Vita Humpa <vhumpa@redhat.com>"""
__version__ = "0.9.10"
__copyright__ = "Copyright © 2005-2017 Red Hat, Inc."
__license__ = "GPL"

__all__ = ("config", "distro", "dump", "errors", "i18n", "logging", "path", "predicate",
           "procedural", "rawinput", "sessions", "tc", "tree", "utils", "version", "wrapped")


__contributor__ = "Mikigo <huangmingqiang@uniontech.com>"

import os
import sys

DEPENDS_PATH = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
sys.path.insert(0, DEPENDS_PATH)



# Windows implementation of PyAutoGUI functions.
# BSD license
# Al Sweigart al@inventwithpython.com

import ctypes
import ctypes.wintypes
import pyautogui
from pyautogui import LEFT, MIDDLE, RIGHT

import sys
if sys.platform !=  'win32':
    raise Exception('The pyautogui_win module should only be loaded on a Windows system.')


# Fixes the scaling issues where PyAutoGUI was reporting the wrong resolution:
try:
   ctypes.windll.user32.SetProcessDPIAware()
except AttributeError:
    pass # Windows XP doesn't support this, so just do nothing.


"""
A lot of this code is probably repeated from win32 extensions module, but I didn't want to have that dependency.

Note: According to http://msdn.microsoft.com/en-us/library/windows/desktop/ms646260(v=vs.85).aspx
the ctypes.windll.user32.mouse_event() function has been superseded by SendInput.

SendInput() is documented here: http://msdn.microsoft.com/en-us/library/windows/desktop/ms646310(v=vs.85).aspx

UPDATE: SendInput() doesn't seem to be working for me. I've switched back to mouse_event()."""


# Event codes to be passed to the mouse_event() win32 function.
# Documented here: http://msdn.microsoft.com/en-us/library/windows/desktop/ms646273(v=vs.85).aspx
MOUSEEVENTF_MOVE = 0x0001
MOUSEEVENTF_LEFTDOWN = 0x0002
MOUSEEVENTF_LEFTUP = 0x0004
MOUSEEVENTF_LEFTCLICK = MOUSEEVENTF_LEFTDOWN + MOUSEEVENTF_LEFTUP
MOUSEEVENTF_RIGHTDOWN = 0x0008
MOUSEEVENTF_RIGHTUP = 0x0010
MOUSEEVENTF_RIGHTCLICK = MOUSEEVENTF_RIGHTDOWN + MOUSEEVENTF_RIGHTUP
MOUSEEVENTF_MIDDLEDOWN = 0x0020
MOUSEEVENTF_MIDDLEUP = 0x0040
MOUSEEVENTF_MIDDLECLICK = MOUSEEVENTF_MIDDLEDOWN + MOUSEEVENTF_MIDDLEUP

MOUSEEVENTF_ABSOLUTE = 0x8000
MOUSEEVENTF_WHEEL = 0x0800
MOUSEEVENTF_HWHEEL = 0x01000

# Documented here: http://msdn.microsoft.com/en-us/library/windows/desktop/ms646304(v=vs.85).aspx
KEYEVENTF_KEYDOWN = 0x0000 # Technically this constant doesn't exist in the MS documentation. It's the lack of KEYEVENTF_KEYUP that means pressing the key down.
KEYEVENTF_KEYUP = 0x0002

# Documented here: http://msdn.microsoft.com/en-us/library/windows/desktop/ms646270(v=vs.85).aspx
INPUT_MOUSE = 0
INPUT_KEYBOARD = 1


# These ctypes structures are for Win32 INPUT, MOUSEINPUT, KEYBDINPUT, and HARDWAREINPUT structures,
# used by SendInput and documented here: http://msdn.microsoft.com/en-us/library/windows/desktop/ms646270(v=vs.85).aspx
# Thanks to BSH for this StackOverflow answer: https://stackoverflow.com/questions/18566289/how-would-you-recreate-this-windows-api-structure-with-ctypes
class MOUSEINPUT(ctypes.Structure):
    _fields_ = [
        ('dx', ctypes.wintypes.LONG),
        ('dy', ctypes.wintypes.LONG),
        ('mouseData', ctypes.wintypes.DWORD),
        ('dwFlags', ctypes.wintypes.DWORD),
        ('time', ctypes.wintypes.DWORD),
        ('dwExtraInfo', ctypes.POINTER(ctypes.wintypes.ULONG)),
    ]

class KEYBDINPUT(ctypes.Structure):
    _fields_ = [
        ('wVk', ctypes.wintypes.WORD),
        ('wScan', ctypes.wintypes.WORD),
        ('dwFlags', ctypes.wintypes.DWORD),
        ('time', ctypes.wintypes.DWORD),
        ('dwExtraInfo', ctypes.POINTER(ctypes.wintypes.ULONG)),
    ]

class HARDWAREINPUT(ctypes.Structure):
    _fields_ = [
        ('uMsg', ctypes.wintypes.DWORD),
        ('wParamL', ctypes.wintypes.WORD),
        ('wParamH', ctypes.wintypes.DWORD)
    ]

class INPUT(ctypes.Structure):
    class _I(ctypes.Union):
        _fields_ = [
            ('mi', MOUSEINPUT),
            ('ki', KEYBDINPUT),
            ('hi', HARDWAREINPUT),
        ]

    _anonymous_ = ('i', )
    _fields_ = [
        ('type', ctypes.wintypes.DWORD),
        ('i', _I),
    ]
# End of the SendInput win32 data structures.



""" Keyboard key mapping for pyautogui:
Documented at http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx

The *KB dictionaries in pyautogui map a string that can be passed to keyDown(),
keyUp(), or press() into the code used for the OS-specific keyboard function.

They should always be lowercase, and the same keys should be used across all OSes."""
keyboardMapping = dict([(key, None) for key in pyautogui.KEY_NAMES])
keyboardMapping.update({
    'backspace': 0x08, # VK_BACK
    '\b': 0x08, # VK_BACK
    'super': 0x5B, #VK_LWIN
    'tab': 0x09, # VK_TAB
    '\t': 0x09, # VK_TAB
    'clear': 0x0c, # VK_CLEAR
    'enter': 0x0d, # VK_RETURN
    '\n': 0x0d, # VK_RETURN
    'return': 0x0d, # VK_RETURN
    'shift': 0x10, # VK_SHIFT
    'ctrl': 0x11, # VK_CONTROL
    'alt': 0x12, # VK_MENU
    'pause': 0x13, # VK_PAUSE
    'capslock': 0x14, # VK_CAPITAL
    'kana': 0x15, # VK_KANA
    'hanguel': 0x15, # VK_HANGUEL
    'hangul': 0x15, # VK_HANGUL
    'junja': 0x17, # VK_JUNJA
    'final': 0x18, # VK_FINAL
    'hanja': 0x19, # VK_HANJA
    'kanji': 0x19, # VK_KANJI
    'esc': 0x1b, # VK_ESCAPE
    'escape': 0x1b, # VK_ESCAPE
    'convert': 0x1c, # VK_CONVERT
    'nonconvert': 0x1d, # VK_NONCONVERT
    'accept': 0x1e, # VK_ACCEPT
    'modechange': 0x1f, # VK_MODECHANGE
    ' ': 0x20, # VK_SPACE
    'space': 0x20, # VK_SPACE
    'pgup': 0x21, # VK_PRIOR
    'pgdn': 0x22, # VK_NEXT
    'pageup': 0x21, # VK_PRIOR
    'pagedown': 0x22, # VK_NEXT
    'end': 0x23, # VK_END
    'home': 0x24, # VK_HOME
    'left': 0x25, # VK_LEFT
    'up': 0x26, # VK_UP
    'right': 0x27, # VK_RIGHT
    'down': 0x28, # VK_DOWN
    'select': 0x29, # VK_SELECT
    'print': 0x2a, # VK_PRINT
    'execute': 0x2b, # VK_EXECUTE
    'prtsc': 0x2c, # VK_SNAPSHOT
    'prtscr': 0x2c, # VK_SNAPSHOT
    'prntscrn': 0x2c, # VK_SNAPSHOT
    'printscreen': 0x2c, # VK_SNAPSHOT
    'insert': 0x2d, # VK_INSERT
    'del': 0x2e, # VK_DELETE
    'delete': 0x2e, # VK_DELETE
    'help': 0x2f, # VK_HELP
    'win': 0x5b, # VK_LWIN
    'winleft': 0x5b, # VK_LWIN
    'winright': 0x5c, # VK_RWIN
    'apps': 0x5d, # VK_APPS
    'sleep': 0x5f, # VK_SLEEP
    'num0': 0x60, # VK_NUMPAD0
    'num1': 0x61, # VK_NUMPAD1
    'num2': 0x62, # VK_NUMPAD2
    'num3': 0x63, # VK_NUMPAD3
    'num4': 0x64, # VK_NUMPAD4
    'num5': 0x65, # VK_NUMPAD5
    'num6': 0x66, # VK_NUMPAD6
    'num7': 0x67, # VK_NUMPAD7
    'num8': 0x68, # VK_NUMPAD8
    'num9': 0x69, # VK_NUMPAD9
    'multiply': 0x6a, # VK_MULTIPLY  ??? Is this the numpad *?
    'add': 0x6b, # VK_ADD  ??? Is this the numpad +?
    'separator': 0x6c, # VK_SEPARATOR  ??? Is this the numpad enter?
    'subtract': 0x6d, # VK_SUBTRACT  ??? Is this the numpad -?
    'decimal': 0x6e, # VK_DECIMAL
    'divide': 0x6f, # VK_DIVIDE
    'f1': 0x70, # VK_F1
    'f2': 0x71, # VK_F2
    'f3': 0x72, # VK_F3
    'f4': 0x73, # VK_F4
    'f5': 0x74, # VK_F5
    'f6': 0x75, # VK_F6
    'f7': 0x76, # VK_F7
    'f8': 0x77, # VK_F8
    'f9': 0x78, # VK_F9
    'f10': 0x79, # VK_F10
    'f11': 0x7a, # VK_F11
    'f12': 0x7b, # VK_F12
    'f13': 0x7c, # VK_F13
    'f14': 0x7d, # VK_F14
    'f15': 0x7e, # VK_F15
    'f16': 0x7f, # VK_F16
    'f17': 0x80, # VK_F17
    'f18': 0x81, # VK_F18
    'f19': 0x82, # VK_F19
    'f20': 0x83, # VK_F20
    'f21': 0x84, # VK_F21
    'f22': 0x85, # VK_F22
    'f23': 0x86, # VK_F23
    'f24': 0x87, # VK_F24
    'numlock': 0x90, # VK_NUMLOCK
    'scrolllock': 0x91, # VK_SCROLL
    'shiftleft': 0xa0, # VK_LSHIFT
    'shiftright': 0xa1, # VK_RSHIFT
    'ctrlleft': 0xa2, # VK_LCONTROL
    'ctrlright': 0xa3, # VK_RCONTROL
    'altleft': 0xa4, # VK_LMENU
    'altright': 0xa5, # VK_RMENU
    'browserback': 0xa6, # VK_BROWSER_BACK
    'browserforward': 0xa7, # VK_BROWSER_FORWARD
    'browserrefresh': 0xa8, # VK_BROWSER_REFRESH
    'browserstop': 0xa9, # VK_BROWSER_STOP
    'browsersearch': 0xaa, # VK_BROWSER_SEARCH
    'browserfavorites': 0xab, # VK_BROWSER_FAVORITES
    'browserhome': 0xac, # VK_BROWSER_HOME
    'volumemute': 0xad, # VK_VOLUME_MUTE
    'volumedown': 0xae, # VK_VOLUME_DOWN
    'volumeup': 0xaf, # VK_VOLUME_UP
    'nexttrack': 0xb0, # VK_MEDIA_NEXT_TRACK
    'prevtrack': 0xb1, # VK_MEDIA_PREV_TRACK
    'stop': 0xb2, # VK_MEDIA_STOP
    'playpause': 0xb3, # VK_MEDIA_PLAY_PAUSE
    'launchmail': 0xb4, # VK_LAUNCH_MAIL
    'launchmediaselect': 0xb5, # VK_LAUNCH_MEDIA_SELECT
    'launchapp1': 0xb6, # VK_LAUNCH_APP1
    'launchapp2': 0xb7, # VK_LAUNCH_APP2
    })

    # There are other virtual key constants that are not used here because the printable ascii keys are
    # handled in the following `for` loop.
    # The virtual key constants that aren't used are:
    # VK_OEM_1, VK_OEM_PLUS, VK_OEM_COMMA, VK_OEM_MINUS, VK_OEM_PERIOD, VK_OEM_2, VK_OEM_3, VK_OEM_4,
    # VK_OEM_5, VK_OEM_6, VK_OEM_7, VK_OEM_8, VK_PACKET, VK_ATTN, VK_CRSEL, VK_EXSEL, VK_EREOF,
    # VK_PLAY, VK_ZOOM, VK_NONAME, VK_PA1, VK_OEM_CLEAR

# Populate the basic printable ascii characters.
# https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-vkkeyscana
for c in range(32, 128):
    keyboardMapping[chr(c)] = ctypes.windll.user32.VkKeyScanA(ctypes.wintypes.WCHAR(chr(c)))


def _keyDown(key):
    """Performs a keyboard key press without the release. This will put that
    key in a held down state.

    NOTE: For some reason, this does not seem to cause key repeats like would
    happen if a keyboard key was held down on a text field.

    Args:
      key (str): The key to be pressed down. The valid names are listed in
      pyautogui.KEY_NAMES.

    Returns:
      None
    """
    if key not in keyboardMapping or keyboardMapping[key] is None:
        return

    needsShift = pyautogui.isShiftCharacter(key)

    """
    # OLD CODE: The new code relies on having all keys be loaded in keyboardMapping from the start.
    if key in keyboardMapping.keys():
        vkCode = keyboardMapping[key]
    elif len(key) == 1:
        # note: I could use this case to update keyboardMapping to cache the VkKeyScan results, but I've decided not to just to make any possible bugs easier to reproduce.
        vkCode = ctypes.windll.user32.VkKeyScanW(ctypes.wintypes.WCHAR(key))
        if vkCode == -1:
            raise ValueError('There is no VK code for key "%s"' % (key))
        if vkCode > 0x100: # the vk code will be > 0x100 if it needs shift
            vkCode -= 0x100
            needsShift = True
    """
    mods, vkCode = divmod(keyboardMapping[key], 0x100)

    for apply_mod, vk_mod in [(mods & 4, 0x12), (mods & 2, 0x11),
        (mods & 1 or needsShift, 0x10)]: #HANKAKU not supported! mods & 8
        if apply_mod:
            ctypes.windll.user32.keybd_event(vk_mod, 0, KEYEVENTF_KEYDOWN, 0) #
    ctypes.windll.user32.keybd_event(vkCode, 0, KEYEVENTF_KEYDOWN, 0)
    for apply_mod, vk_mod in [(mods & 1 or needsShift, 0x10), (mods & 2, 0x11),
        (mods & 4, 0x12)]: #HANKAKU not supported! mods & 8
        if apply_mod:
            ctypes.windll.user32.keybd_event(vk_mod, 0, KEYEVENTF_KEYUP, 0) #


def _keyUp(key):
    """Performs a keyboard key release (without the press down beforehand).

    Args:
      key (str): The key to be released up. The valid names are listed in
      pyautogui.KEY_NAMES.

    Returns:
      None
    """
    if key not in keyboardMapping or keyboardMapping[key] is None:
        return

    needsShift = pyautogui.isShiftCharacter(key)
    """
    # OLD CODE: The new code relies on having all keys be loaded in keyboardMapping from the start.
    if key in keyboardMapping.keys():
        vkCode = keyboardMapping[key]
    elif len(key) == 1:
        # note: I could use this case to update keyboardMapping to cache the VkKeyScan results, but I've decided not to just to make any possible bugs easier to reproduce.
        vkCode = ctypes.windll.user32.VkKeyScanW(ctypes.wintypes.WCHAR(key))
        if vkCode == -1:
            raise ValueError('There is no VK code for key "%s"' % (key))
        if vkCode > 0x100: # the vk code will be > 0x100 if it needs shift
            vkCode -= 0x100
            needsShift = True
    """
    mods, vkCode = divmod(keyboardMapping[key], 0x100)

    for apply_mod, vk_mod in [(mods & 4, 0x12), (mods & 2, 0x11),
        (mods & 1 or needsShift, 0x10)]: #HANKAKU not supported! mods & 8
        if apply_mod:
            ctypes.windll.user32.keybd_event(vk_mod, 0, 0, 0) #
    ctypes.windll.user32.keybd_event(vkCode, 0, KEYEVENTF_KEYUP, 0)
    for apply_mod, vk_mod in [(mods & 1 or needsShift, 0x10), (mods & 2, 0x11),
        (mods & 4, 0x12)]: #HANKAKU not supported! mods & 8
        if apply_mod:
            ctypes.windll.user32.keybd_event(vk_mod, 0, KEYEVENTF_KEYUP, 0) #


def _position():
    """Returns the current xy coordinates of the mouse cursor as a two-integer
    tuple by calling the GetCursorPos() win32 function.

    Returns:
      (x, y) tuple of the current xy coordinates of the mouse cursor.
    """

    cursor = ctypes.wintypes.POINT()
    ctypes.windll.user32.GetCursorPos(ctypes.byref(cursor))
    return (cursor.x, cursor.y)


def _size():
    """Returns the width and height of the screen as a two-integer tuple.

    Returns:
      (width, height) tuple of the screen size, in pixels.
    """
    return (ctypes.windll.user32.GetSystemMetrics(0), ctypes.windll.user32.GetSystemMetrics(1))


def _moveTo(x, y):
    """Send the mouse move event to Windows by calling SetCursorPos() win32
    function.

    Args:
      button (str): The mouse button, either 'left', 'middle', or 'right'
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.

    Returns:
      None
    """
    ctypes.windll.user32.SetCursorPos(x, y)
    # This was a possible solution to issue #314 https://github.com/asweigart/pyautogui/issues/314
    # but I'd like to hang on to SetCursorPos because mouse_event() has been superseded.
    #_sendMouseEvent(MOUSEEVENTF_MOVE + MOUSEEVENTF_ABSOLUTE, x, y)


def _mouseDown(x, y, button):
    """Send the mouse down event to Windows by calling the mouse_event() win32
    function.

    Args:
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.
      button (str): The mouse button, either 'left', 'middle', or 'right'

    Returns:
      None
    """
    if button not in (LEFT, MIDDLE, RIGHT):
        raise ValueError('button arg to _click() must be one of "left", "middle", or "right", not %s' % button)

    if button == LEFT:
        EV = MOUSEEVENTF_LEFTDOWN
    elif button == MIDDLE:
        EV = MOUSEEVENTF_MIDDLEDOWN
    elif button == RIGHT:
        EV = MOUSEEVENTF_RIGHTDOWN

    try:
        _sendMouseEvent(EV, x, y)
    except (PermissionError, OSError):
        # TODO: We need to figure out how to prevent these errors, see https://github.com/asweigart/pyautogui/issues/60
        pass


def _mouseUp(x, y, button):
    """Send the mouse up event to Windows by calling the mouse_event() win32
    function.

    Args:
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.
      button (str): The mouse button, either 'left', 'middle', or 'right'

    Returns:
      None
    """
    if button not in (LEFT, MIDDLE, RIGHT):
        raise ValueError('button arg to _click() must be one of "left", "middle", or "right", not %s' % button)

    if button == LEFT:
        EV = MOUSEEVENTF_LEFTUP
    elif button == MIDDLE:
        EV = MOUSEEVENTF_MIDDLEUP
    elif button == RIGHT:
        EV = MOUSEEVENTF_RIGHTUP

    try:
        _sendMouseEvent(EV, x, y)
    except (PermissionError, OSError): # TODO: We need to figure out how to prevent these errors, see https://github.com/asweigart/pyautogui/issues/60
        pass


def _click(x, y, button):
    """Send the mouse click event to Windows by calling the mouse_event() win32
    function.

    Args:
      button (str): The mouse button, either 'left', 'middle', or 'right'
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.

    Returns:
      None
    """
    if button not in (LEFT, MIDDLE, RIGHT):
        raise ValueError('button arg to _click() must be one of "left", "middle", or "right", not %s' % button)

    if button == LEFT:
        EV = MOUSEEVENTF_LEFTCLICK
    elif button == MIDDLE:
        EV = MOUSEEVENTF_MIDDLECLICK
    elif button ==RIGHT:
        EV = MOUSEEVENTF_RIGHTCLICK

    try:
        _sendMouseEvent(EV, x, y)
    except (PermissionError, OSError):
        # TODO: We need to figure out how to prevent these errors, see https://github.com/asweigart/pyautogui/issues/60
        pass


def _sendMouseEvent(ev, x, y, dwData=0):
    """The helper function that actually makes the call to the mouse_event()
    win32 function.

    Args:
      ev (int): The win32 code for the mouse event. Use one of the MOUSEEVENTF_*
      constants for this argument.
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.
      dwData (int): The argument for mouse_event()'s dwData parameter. So far
        this is only used by mouse scrolling.

    Returns:
      None
    """
    assert x != None and y != None, 'x and y cannot be set to None'
    # TODO: ARG! For some reason, SendInput isn't working for mouse events. I'm switching to using the older mouse_event win32 function.
    #mouseStruct = MOUSEINPUT()
    #mouseStruct.dx = x
    #mouseStruct.dy = y
    #mouseStruct.mouseData = ev
    #mouseStruct.time = 0
    #mouseStruct.dwExtraInfo = ctypes.pointer(ctypes.c_ulong(0)) # according to https://stackoverflow.com/questions/13564851/generate-keyboard-events I can just set this. I don't really care about this value.
    #inputStruct = INPUT()
    #inputStruct.mi = mouseStruct
    #inputStruct.type = INPUT_MOUSE
    #ctypes.windll.user32.SendInput(1, ctypes.pointer(inputStruct), ctypes.sizeof(inputStruct))

    # TODO Note: We need to handle additional buttons, which I believe is documented here:
    # https://docs.microsoft.com/en-us/windows/desktop/api/winuser/nf-winuser-mouse_event

    width, height = _size()
    convertedX = 65536 * x // width + 1
    convertedY = 65536 * y // height + 1
    ctypes.windll.user32.mouse_event(ev, ctypes.c_long(convertedX), ctypes.c_long(convertedY), dwData, 0)

    # TODO: Too many false positives with this code: See: https://github.com/asweigart/pyautogui/issues/108
    #if ctypes.windll.kernel32.GetLastError() != 0:
    #    raise ctypes.WinError()


def _scroll(clicks, x=None, y=None):
    """Send the mouse vertical scroll event to Windows by calling the
    mouse_event() win32 function.

    Args:
      clicks (int): The amount of scrolling to do. A positive value is the mouse
      wheel moving forward (scrolling up), a negative value is backwards (down).
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.

    Returns:
      None
    """
    startx, starty = _position()
    width, height = _size()

    if x is None:
        x = startx
    else:
        if x < 0:
            x = 0
        elif x >= width:
            x = width - 1
    if y is None:
        y = starty
    else:
        if y < 0:
            y = 0
        elif y >= height:
            y = height - 1

    try:
        _sendMouseEvent(MOUSEEVENTF_WHEEL, x, y, dwData=clicks)
    except (PermissionError, OSError): # TODO: We need to figure out how to prevent these errors, see https://github.com/asweigart/pyautogui/issues/60
            pass


def _hscroll(clicks, x, y):
    """Send the mouse horizontal scroll event to Windows by calling the
    mouse_event() win32 function.

    Args:
      clicks (int): The amount of scrolling to do. A positive value is the mouse
      wheel moving right, a negative value is moving left.
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.

    Returns:
      None
    """
    return _scroll(clicks, x, y)


def _vscroll(clicks, x, y):
    """A wrapper for _scroll(), which does vertical scrolling.

    Args:
      clicks (int): The amount of scrolling to do. A positive value is the mouse
      wheel moving forward (scrolling up), a negative value is backwards (down).
      x (int): The x position of the mouse event.
      y (int): The y position of the mouse event.

    Returns:
      None
    """
    return _scroll(clicks, x, y)




# Copyright Jonathan Hartley 2013. BSD 3-Clause license, see LICENSE file.
import atexit
import contextlib
import sys

from .ansitowin32 import AnsiToWin32


def _wipe_internal_state_for_tests():
    global orig_stdout, orig_stderr
    orig_stdout = None
    orig_stderr = None

    global wrapped_stdout, wrapped_stderr
    wrapped_stdout = None
    wrapped_stderr = None

    global atexit_done
    atexit_done = False

    global fixed_windows_console
    fixed_windows_console = False

    try:
        # no-op if it wasn't registered
        atexit.unregister(reset_all)
    except AttributeError:
        # python 2: no atexit.unregister. Oh well, we did our best.
        pass


def reset_all():
    if AnsiToWin32 is not None:    # Issue #74: objects might become None at exit
        AnsiToWin32(orig_stdout).reset_all()


def init(autoreset=False, convert=None, strip=None, wrap=True):

    if not wrap and any([autoreset, convert, strip]):
        raise ValueError('wrap=False conflicts with any other arg=True')

    global wrapped_stdout, wrapped_stderr
    global orig_stdout, orig_stderr

    orig_stdout = sys.stdout
    orig_stderr = sys.stderr

    if sys.stdout is None:
        wrapped_stdout = None
    else:
        sys.stdout = wrapped_stdout = \
            wrap_stream(orig_stdout, convert, strip, autoreset, wrap)
    if sys.stderr is None:
        wrapped_stderr = None
    else:
        sys.stderr = wrapped_stderr = \
            wrap_stream(orig_stderr, convert, strip, autoreset, wrap)

    global atexit_done
    if not atexit_done:
        atexit.register(reset_all)
        atexit_done = True


def deinit():
    if orig_stdout is not None:
        sys.stdout = orig_stdout
    if orig_stderr is not None:
        sys.stderr = orig_stderr


def just_fix_windows_console():
    global fixed_windows_console

    if sys.platform != "win32":
        return
    if fixed_windows_console:
        return
    if wrapped_stdout is not None or wrapped_stderr is not None:
        # Someone already ran init() and it did stuff, so we won't second-guess them
        return

    # On newer versions of Windows, AnsiToWin32.__init__ will implicitly enable the
    # native ANSI support in the console as a side-effect. We only need to actually
    # replace sys.stdout/stderr if we're in the old-style conversion mode.
    new_stdout = AnsiToWin32(sys.stdout, convert=None, strip=None, autoreset=False)
    if new_stdout.convert:
        sys.stdout = new_stdout
    new_stderr = AnsiToWin32(sys.stderr, convert=None, strip=None, autoreset=False)
    if new_stderr.convert:
        sys.stderr = new_stderr

    fixed_windows_console = True

@contextlib.contextmanager
def colorama_text(*args, **kwargs):
    init(*args, **kwargs)
    try:
        yield
    finally:
        deinit()


def reinit():
    if wrapped_stdout is not None:
        sys.stdout = wrapped_stdout
    if wrapped_stderr is not None:
        sys.stderr = wrapped_stderr


def wrap_stream(stream, convert, strip, autoreset, wrap):
    if wrap:
        wrapper = AnsiToWin32(stream,
            convert=convert, strip=strip, autoreset=autoreset)
        if wrapper.should_wrap():
            stream = wrapper.stream
    return stream


# Use this for initial setup as well, to reduce code duplication
_wipe_internal_state_for_tests()



from src.requestx import RequestX


class ApiClient:
    def __init__(self, base_url, username, password):
        self.base_url = base_url
        self.username = username
        self.password = password
        self.token = self.get_token()
        self.header = self.get_header()

    def get_token(self):
        # 设置身份验证端点URL
        auth_endpoint = f"{self.base_url}/api/token/"

        # 设置用户凭据
        credentials = {
            "username": self.username,
            "password": self.password
        }
        headers = {"content-type": "application/x-www-form-urlencoded; charset=UTF-8"}
        res = RequestX.post(url=auth_endpoint, data=credentials, headers=headers)
        import json
        jwt_token = json.loads(res)["data"]["access"]
        return jwt_token

    def get_header(self):
        header = {
            "Authorization": f"JWT {self.token}",
            "Content-Type": "application/json"
        }
        return header

    def post(self, url, data=None, json=None):
        response = RequestX.post(url, headers=self.header, data=data, json=json)
        return response


if __name__ == "__main__":
    ApiClient(
        base_url="http://10.7.55.191:8000",
        username="",
        password="",
    )



"""
    Python cFonts
    =============
    Sexy fonts for the console.

    :license: GNU GPLv2
    :author: Frost Ming<mianghong@gmail.com>
"""
import enum
from shutil import get_terminal_size
from typing import Mapping, Tuple

SIZE = get_terminal_size((80, 24))
CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789|!?.+-_=@#$%&()/:;,' \""


class Colors(enum.Enum):
    system = "system"
    black = "black"
    red = "red"
    green = "green"
    yellow = "yellow"
    blue = "blue"
    magenta = "magenta"
    cyan = "cyan"
    white = "white"
    candy = "candy"
    bright_red = "bright_red"
    bright_green = "bright_green"
    bright_yellow = "bright_yellow"
    bright_blue = "bright_blue"
    bright_magenta = "bright_magenta"
    bright_cyan = "bright_cyan"
    bright_white = "bright_white"


class CandyColors(enum.Enum):
    red = "red"
    green = "green"
    yellow = "yellow"
    blue = "blue"
    magenta = "magenta"
    cyan = "cyan"
    gray = "gray"
    bright_red = "bright_red"
    bright_green = "bright_green"
    bright_yellow = "bright_yellow"
    bright_blue = "bright_blue"
    bright_magenta = "bright_magenta"
    bright_cyan = "bright_cyan"


class BgColors(enum.Enum):
    transparent = "transparent"
    black = "black"
    red = "red"
    green = "green"
    yellow = "yellow"
    blue = "blue"
    magenta = "magenta"
    cyan = "cyan"
    white = "white"
    bright_black = "bright_black"
    bright_red = "bright_red"
    bright_green = "bright_green"
    bright_yellow = "bright_yellow"
    bright_blue = "bright_blue"
    bright_magenta = "bright_magenta"
    bright_cyan = "bright_cyan"
    bright_white = "bright_white"


ALIGNMENT = ["left", "center", "right"]


class FontFaces(enum.Enum):
    console = "console"
    block = "block"
    simpleblock = "simpleBlock"
    simple = "simple"
    threed = "3d"
    simple3d = "simple3d"
    chrome = "chrome"
    huge = "huge"
    grid = "grid"
    pallet = "pallet"
    shade = "shade"
    slick = "slick"
    tiny = "tiny"


ANSI_COLORS = {
    "black": 30,
    "red": 31,
    "green": 32,
    "yellow": 33,
    "blue": 34,
    "magenta": 35,
    "cyan": 36,
    "white": 37,
    "gray": 90,
    "bright_red": 91,
    "bright_green": 92,
    "bright_yellow": 93,
    "bright_blue": 94,
    "bright_magenta": 95,
    "bright_cyan": 96,
    "bright_white": 97,
}

ANSI_RGB: Mapping[str, Tuple[int, int, int]] = {
    "black": (0, 0, 0),
    "red": (234, 50, 35),
    "green": (55, 125, 34),
    "yellow": (255, 253, 84),
    "blue": (0, 32, 245),
    "magenta": (234, 61, 247),
    "cyan": (116, 251, 253),
    "white": (255, 255, 255),
    "gray": (128, 128, 128),
    "bright_red": (238, 119, 109),
    "bright_green": (140, 245, 123),
    "bright_yellow": (255, 251, 127),
    "bright_blue": (105, 116, 246),
    "bright_magenta": (238, 130, 248),
    "bright_cyan": (141, 250, 253),
    "bright_white": (255, 255, 255),
}



# NOTE - It is a known issue that the keyboard-related functions don't work on Ubuntu VMs in Virtualbox.

import sys
import os
from src.depends import pyautogui
from src.depends.pyautogui import LEFT, MIDDLE, RIGHT

try:
    from Xlib.display import Display
    from Xlib import X
    from Xlib.ext.xtest import fake_input
    import Xlib.XK
except ModuleNotFoundError:
    pass

BUTTON_NAME_MAPPING = {LEFT: 1, MIDDLE: 2, RIGHT: 3, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7}

if sys.platform in ('java', 'darwin', 'win32'):
    raise Exception('The pyautogui_x11 module should only be loaded on a Unix system that supports X11.')

# from pyautogui import *

"""
Much of this code is based on information gleaned from Paul Barton's PyKeyboard in PyUserInput from 2013, itself derived from Akkana Peck's pykey in 2008 ( http://www.shallowsky.com/software/crikey/pykey-0.1 ), itself derived from her "Crikey" lib.
"""


def _position():
    """Returns the current xy coordinates of the mouse cursor as a two-integer
    tuple.

    Returns:
      (x, y) tuple of the current xy coordinates of the mouse cursor.
    """
    try:
        coord = _display.screen().root.query_pointer()._data
        return coord["root_x"], coord["root_y"]
    except NameError:
        return (1, 1)


def _size():
    try:
        return _display.screen().width_in_pixels, _display.screen().height_in_pixels
    except NameError:
        return (1, 1)


def _vscroll(clicks, x=None, y=None):
    clicks = int(clicks)
    if clicks == 0:
        return
    elif clicks > 0:
        button = 4  # scroll up
    else:
        button = 5  # scroll down

    for i in range(abs(clicks)):
        _click(x, y, button=button)


def _hscroll(clicks, x=None, y=None):
    clicks = int(clicks)
    if clicks == 0:
        return
    elif clicks > 0:
        button = 7  # scroll right
    else:
        button = 6  # scroll left

    for i in range(abs(clicks)):
        _click(x, y, button=button)


def _scroll(clicks, x=None, y=None):
    return _vscroll(clicks, x, y)


def _click(x, y, button):
    assert button in BUTTON_NAME_MAPPING.keys(), "button argument not in ('left', 'middle', 'right', 4, 5, 6, 7)"
    button = BUTTON_NAME_MAPPING[button]

    _mouseDown(x, y, button)
    _mouseUp(x, y, button)


def _moveTo(x, y):
    fake_input(_display, X.MotionNotify, x=x, y=y)
    _display.sync()


def _mouseDown(x, y, button):
    _moveTo(x, y)
    assert button in BUTTON_NAME_MAPPING.keys(), "button argument not in ('left', 'middle', 'right', 4, 5, 6, 7)"
    button = BUTTON_NAME_MAPPING[button]
    fake_input(_display, X.ButtonPress, button)
    _display.sync()


def _mouseUp(x, y, button):
    _moveTo(x, y)
    assert button in BUTTON_NAME_MAPPING.keys(), "button argument not in ('left', 'middle', 'right', 4, 5, 6, 7)"
    button = BUTTON_NAME_MAPPING[button]
    fake_input(_display, X.ButtonRelease, button)
    _display.sync()


def _keyDown(key):
    """Performs a keyboard key press without the release. This will put that
    key in a held down state.

    NOTE: For some reason, this does not seem to cause key repeats like would
    happen if a keyboard key was held down on a text field.

    Args:
      key (str): The key to be pressed down. The valid names are listed in
      pyautogui.KEY_NAMES.

    Returns:
      None
    """
    if key not in keyboardMapping or keyboardMapping[key] is None:
        return

    if type(key) == int:
        fake_input(_display, X.KeyPress, key)
        _display.sync()
        return

    needsShift = pyautogui.isShiftCharacter(key)
    if needsShift:
        fake_input(_display, X.KeyPress, keyboardMapping['shift'])

    fake_input(_display, X.KeyPress, keyboardMapping[key])

    if needsShift:
        fake_input(_display, X.KeyRelease, keyboardMapping['shift'])
    _display.sync()


def _keyUp(key):
    """Performs a keyboard key release (without the press down beforehand).

    Args:
      key (str): The key to be released up. The valid names are listed in
      pyautogui.KEY_NAMES.

    Returns:
      None
    """

    """
    Release a given character key. Also works with character keycodes as
    integers, but not keysyms.
    """
    if key not in keyboardMapping or keyboardMapping[key] is None:
        return

    if type(key) == int:
        keycode = key
    else:
        keycode = keyboardMapping[key]

    fake_input(_display, X.KeyRelease, keycode)
    _display.sync()

try:
    # Taken from PyKeyboard's ctor function.
    _display = Display(os.environ['DISPLAY'])

    """ Information for keyboardMapping derived from PyKeyboard's special_key_assignment() function.
    
    The *KB dictionaries in pyautogui map a string that can be passed to keyDown(),
    keyUp(), or press() into the code used for the OS-specific keyboard function.
    
    They should always be lowercase, and the same keys should be used across all OSes."""
    keyboardMapping = dict([(key, None) for key in pyautogui.KEY_NAMES])
    keyboardMapping.update({
        'backspace': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('BackSpace')),
        '\b': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('BackSpace')),
        'tab': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Tab')),
        'enter': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Return')),
        'return': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Return')),
        'shift': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Shift_L')),
        'ctrl': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Control_L')),
        'alt': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Alt_L')),
        'pause': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Pause')),
        'capslock': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Caps_Lock')),
        'esc': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Escape')),
        'escape': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Escape')),
        'pgup': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Page_Up')),
        'pgdn': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Page_Down')),
        'pageup': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Page_Up')),
        'pagedown': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Page_Down')),
        'end': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('End')),
        'home': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Home')),
        'left': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Left')),
        'up': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Up')),
        'right': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Right')),
        'down': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Down')),
        'select': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Select')),
        'print': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Print')),
        'execute': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Execute')),
        'prtsc': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Print')),
        'prtscr': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Print')),
        'prntscrn': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Print')),
        'printscreen': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Print')),
        'insert': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Insert')),
        'del': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Delete')),
        'delete': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Delete')),
        'help': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Help')),
        'win': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Super_L')),
        'winleft': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Super_L')),
        'winright': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Super_R')),
        'apps': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Menu')),
        'num0': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_0')),
        'num1': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_1')),
        'num2': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_2')),
        'num3': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_3')),
        'num4': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_4')),
        'num5': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_5')),
        'num6': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_6')),
        'num7': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_7')),
        'num8': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_8')),
        'num9': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_9')),
        'multiply': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_Multiply')),
        'add': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_Add')),
        'separator': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_Separator')),
        'subtract': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_Subtract')),
        'decimal': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_Decimal')),
        'divide': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('KP_Divide')),
        'f1': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F1')),
        'f2': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F2')),
        'f3': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F3')),
        'f4': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F4')),
        'f5': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F5')),
        'f6': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F6')),
        'f7': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F7')),
        'f8': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F8')),
        'f9': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F9')),
        'f10': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F10')),
        'f11': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F11')),
        'f12': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F12')),
        'f13': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F13')),
        'f14': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F14')),
        'f15': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F15')),
        'f16': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F16')),
        'f17': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F17')),
        'f18': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F18')),
        'f19': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F19')),
        'f20': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F20')),
        'f21': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F21')),
        'f22': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F22')),
        'f23': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F23')),
        'f24': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('F24')),
        'numlock': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Num_Lock')),
        'scrolllock': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Scroll_Lock')),
        'shiftleft': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Shift_L')),
        'shiftright': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Shift_R')),
        'ctrlleft': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Control_L')),
        'ctrlright': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Control_R')),
        'altleft': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Alt_L')),
        'altright': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Alt_R')),
        # These are added because unlike a-zA-Z0-9, the single characters do not have a
        ' ': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('space')),
        'space': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('space')),
        '\t': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Tab')),
        '\n': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Return')),  # for some reason this needs to be cr, not lf
        '\r': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Return')),
        '\e': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('Escape')),
        '!': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('exclam')),
        '#': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('numbersign')),
        '%': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('percent')),
        '$': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('dollar')),
        '&': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('ampersand')),
        '"': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('quotedbl')),
        "'": _display.keysym_to_keycode(Xlib.XK.string_to_keysym('apostrophe')),
        '(': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('parenleft')),
        ')': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('parenright')),
        '*': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('asterisk')),
        '=': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('equal')),
        '+': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('plus')),
        ',': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('comma')),
        '-': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('minus')),
        '.': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('period')),
        '/': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('slash')),
        ':': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('colon')),
        ';': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('semicolon')),
        '<': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('less')),
        '>': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('greater')),
        '?': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('question')),
        '@': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('at')),
        '[': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('bracketleft')),
        ']': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('bracketright')),
        '\\': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('backslash')),
        '^': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('asciicircum')),
        '_': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('underscore')),
        '`': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('grave')),
        '{': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('braceleft')),
        '|': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('bar')),
        '}': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('braceright')),
        '~': _display.keysym_to_keycode(Xlib.XK.string_to_keysym('asciitilde')),
    })

    # Trading memory for time" populate winKB so we don't have to call VkKeyScanA each time.
    for c in """abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890""":
        keyboardMapping[c] = _display.keysym_to_keycode(Xlib.XK.string_to_keysym(c))
except (NameError, AttributeError, Xlib.error.DisplayConnectionError):
    pass


#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import json
import os
from copy import deepcopy
from datetime import datetime

from setting import conf
from src.depends.colorama import Fore
from src.git.commit import Commit
from src.rtk._base import transform_app_name
from src import logger


class CodeStatistics(Commit):
    __author__ = "mikigo<huangmingqiang@uniontech.com>"

    def __init__(
            self,
            app_name: str,
            branch: str,
            startdate: str = None,
            enddate: str = None,
            **kwargs,
    ):
        self.app_name = transform_app_name(app_name or conf.APP_NAME)
        self.repo_path = f"{conf.APPS_PATH}/{self.app_name}"
        self.branch = branch or conf.BRANCH
        self.startdate = startdate or conf.START_DATE

        if startdate:
            super().__init__(app_name, branch=branch, startdate=startdate, enddate=enddate)

        self.ignore_txt = ["from", "import", '"""', '    """', "class", "#", "@"]

        for i in range(100):
            self.ignore_txt.append(" " * i + "#")

    def get_git_files(self, commit_id, max_width=10000):
        git_files = []
        _files = os.popen(f"cd {self.repo_path};git show --stat={max_width} {commit_id}").read()
        files = _files.split("\n")
        for file_info in files:
            if "| " not in file_info:
                continue
            file_path = file_info.split("| ")[0].replace(" ", "")
            if file_path.endswith(".py") and not file_path.split("/")[-1].startswith("__"):
                git_files.append(file_path)
        return git_files

    def compare_files(self, commit_id, author, git_dt: datetime):
        _fix_debug = []
        _new_debug = []
        new_test_case_num = 0
        del_test_case_num = 0
        fix_test_case_num = 0
        new_method_num = 0
        del_method_num = 0
        fix_method_num = 0
        new_test_cases = []
        del_test_cases = []
        fix_test_cases = []
        new_mehthods = []
        del_mehthods = []
        fix_mehthods = []
        git_files = self.get_git_files(commit_id=commit_id)
        for filepath in git_files:
            filename = filepath.split("/")[-1]
            dif_texts = os.popen(f"cd {self.repo_path}/;git show {commit_id} -- {filepath}").read()
            logger.info(Fore.GREEN + ("=" * 100) + Fore.RESET)
            logger.info(filepath + "\n" + dif_texts)
            dif_lines = dif_texts.splitlines()
            # case
            if filename.startswith("test_"):
                for line in dif_lines:
                    if line.startswith("--- /dev/null"):
                        new_test_case_num += 1
                        new_test_cases.append(filepath)
                        break
                    elif line.startswith("+++ /dev/null"):
                        del_test_case_num += 1
                        del_test_cases.append(filepath)
                        break
                else:
                    fix_test_case_num += 1
                    fix_test_cases.append(filepath)
            # method
            else:
                methods = []
                method_info = {}
                for line in dif_lines:
                    if line.startswith(("---", "+++", "@@", "@")):
                        continue
                    if "def " in line:
                        if method_info:
                            methods.append(method_info)
                            method_info = {}
                        method_info["name"] = line
                    else:
                        if method_info.get("method_content"):
                            method_info["method_content"].append(line)
                        else:
                            method_info["method_content"] = [line]
                methods.append(method_info)
                for method in methods:
                    method_name = method.get("name")
                    method_content = method.get("method_content")

                    if method_name is None:
                        if method_content:
                            for content in method_content:
                                if content.startswith(("-", "+")):
                                    if any([
                                        content[1:].startswith(tuple(self.ignore_txt)),
                                        content[1:] == "",
                                        content[1:].endswith(('"""', "@staticmethod", "@classmethod"))
                                    ]):
                                        continue
                                    _fix_debug.append(method)
                                    fix_method_num += 1
                                    fix_mehthods.append(f"{filepath}[{method_name}]")
                                    break

                    # 正常出现的方法
                    # 方法名称是+开头，直接视为新增方法
                    elif method_name.startswith("+"):
                        _new_debug.append(method)
                        new_method_num += 1
                        new_mehthods.append(f"{filepath}[{method_name}]")
                    # 方法名称是-开头，直接视为删除方法
                    elif method_name.startswith("-"):
                        del_method_num += 1
                        del_mehthods.append(f"{filepath}[{method_name}]")
                    else:
                        if method_content:
                            for content in method_content:
                                if content.startswith(("-", "+")):
                                    if any([
                                        content[1:] == "",
                                        content[1:].endswith(('"""', "@staticmethod", "@classmethod"))
                                    ]):
                                        continue
                                    _fix_debug.append(method)
                                    fix_method_num += 1
                                    fix_mehthods.append(f"{filepath}[{method_name}]")
                                    break

        res = {
            "commit_id": commit_id,
            "author": author,
            "git_dt": git_dt.strftime("%Y-%m-%d"),
            "branch": self.branch,
            "新增用例": new_test_case_num,
            "删除用例": del_test_case_num,
            "修改用例": fix_test_case_num,
            "新增方法": new_method_num,
            "删除方法": del_method_num,
            "修改方法": fix_method_num,
            "新增用例明细": new_test_cases,
            "删除用例明细": del_test_cases,
            "修复用例明细": fix_test_cases,
            "新增方法明细": new_mehthods,
            "删除方法明细": del_mehthods,
            "修复方法明细": fix_mehthods,
        }
        return res

    def write_result(self, res, detail=False):
        if not os.path.exists(conf.REPORT_PATH):
            os.makedirs(conf.REPORT_PATH)
        result_file = os.path.join(
            conf.REPORT_PATH,
            f"{self.app_name}_git_commit_result{f'_detail' if detail else ''}.json",
        )
        with open(result_file, "w", encoding="utf-8") as f:
            f.write(json.dumps(res, ensure_ascii=False, indent=4, default=None))

        with open(result_file, "r", encoding="utf-8") as f:
            logger.info(f.read())
        logger.info(f"{Fore.GREEN}数据结果{'详细' if detail else '汇总'}报告：{result_file}{Fore.RESET}")

    def codex(self):
        if not self.startdate:
            raise ValueError

        results_detail = []
        commit_id_pairs = self.commit_id()
        results = {}
        for i, (commit_id, _author, git_dt, msg) in enumerate(commit_id_pairs):
            _res = self.compare_files(commit_id, _author, git_dt)
            res = deepcopy(_res)
            _res["msg"] = "\n".join(msg)
            results_detail.append(_res)
            author = res["author"]
            new_test_case_num = res["新增用例"]
            del_test_case_num = res["删除用例"]
            fix_test_case_num = res["修改用例"]
            new_method_num = res["新增方法"]
            del_method_num = res["删除方法"]
            fix_method_num = res["修改方法"]
            commit_id = res["commit_id"]
            _git_dt = res["git_dt"]
            if results.get(author) is None:
                results[author] = res
            else:
                results[author]["新增用例"] += new_test_case_num
                results[author]["删除用例"] += del_test_case_num
                results[author]["修改用例"] += fix_test_case_num
                results[author]["新增方法"] += new_method_num
                results[author]["删除方法"] += del_method_num
                results[author]["修改方法"] += fix_method_num
                results[author]["commit_id"] = commit_id
                results[author]["git_dt_end"] = _git_dt

        if results is None:
            raise ValueError
        logger.info(Fore.GREEN +  ("=" * 100) + Fore.RESET)
        self.write_result(results)
        if results_detail:
            self.write_result(results_detail, detail=True)


if __name__ == "__main__":
    app_name = "apps/autotest_kernel"
    CodeStatistics(
        app_name=app_name,
        branch="at-develop/eagle",
        startdate="2024-01-01",
        # enddate="2024-02-23",
    ).codex()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
import os

py_debs = []
for root, dirs, files in os.walk(os.path.abspath("../../apps")):
    for file in files:
        if file == "requirement_deb.txt":
            with open(f"{root}/{file}", "r", encoding="utf-8") as f:
                py_debs.extend([i.strip() for i in f.read().split("\n")])

print(" ".join(py_debs), end="")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=C0301,C0116,W0613,W1514,C0103,C0103
import json
import os
import re
from urllib.parse import urljoin

from setting import conf
from src.rtk.api_client import ApiClient


class JsonBackfill:
    __author__ = "huangmingqiang@uniontech.com"

    def __init__(self, base_url, username, password, custom_api="api/youqu/yqresult/"):
        self.base_url = base_url
        self.username = username
        self.password = password
        self.custom_api = custom_api
        self.api = ApiClient(
            base_url=self.base_url,
            username=self.username,
            password=self.password,
        )

    def get_remote_json_data(self, json_path):
        json_res = {}
        for file in os.listdir(json_path):
            if file.startswith("detail_report_") and file.endswith(".json"):
                client_ip = re.findall(r"detail_report_(\d+\.\d+\.\d+\.\d+)\.json", file)
                file_path = os.path.join(json_path, file)
                with open(file_path, "r", encoding="utf-8") as f:
                    json_data = json.load(f)
                    if client_ip:
                        json_res[client_ip[0]] = json_data
        return json_res

    def remote_backfill(self, json_path, json_backfill_task_id):
        json_res = self.get_remote_json_data(json_path)
        tpl = {
            "case": "",
            "task": json_backfill_task_id,
            "result": "",
            "is_closed": "",
            "module": "",
            "longrepr": "",
            "pm_ip": "",
            "owner": "",
        }
        for _ip, res in json_res.items():
            for case_py_path, value in res.items():
                _, module, *_, case_py = case_py_path.split("/")
                tpl["case"] = case_py
                case_id = re.findall(r"test_.*?_(\d+)", case_py)
                if case_id:
                    tpl["case"] = case_id[0]
                tpl["module"] = module
                result = value.get("result")
                tpl["result"] = result
                tpl["is_closed"] = False if result == "fail" else True
                tpl["longrepr"] = value.get("longrepr")
                tpl["pm_ip"] = _ip
                res = self.api.post(
                    url=urljoin(self.base_url, self.custom_api),
                    json=tpl
                )
                print(self.custom_api, res)


if __name__ == '__main__':
    JsonBackfill(
        base_url="http://10.7.55.191:8000",
        username="",
        password="",
    ).remote_backfill(f"{conf.REPORT_PATH}/json/0524上午113458_remote/", "d3439082af374a94b95e1d3c0613f513")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import sys
from os.path import dirname
from os.path import basename
from os.path import abspath
from os import environ

sys.path.append(dirname(dirname(dirname(abspath(__file__)))))

from setting import conf

environ["XAUTHORITY"] = f"{conf.HOME}/.Xauthority"
from src.remotectl._base import check_rpc_started as check_rpc_started
from src.remotectl._base import remote_client as remote_client
from src.remotectl._base import remote_server as remote_server

dogtail_port = 4242


@check_rpc_started(basename(__file__))
def remote_dogtail_ctl(user=None, ip=None, password=None, restart_service=False):
    return remote_client(ip=ip, port=dogtail_port)


if __name__ == "__main__":
    from src.dogtail_utils import DogtailUtils

    remote_server(DogtailUtils(), dogtail_port)



import time
import sys

try:
    import Quartz
except:
    assert False, "You must first install pyobjc-core and pyobjc: https://pyautogui.readthedocs.io/en/latest/install.html"
import AppKit

import pyautogui
from pyautogui import LEFT, MIDDLE, RIGHT

if sys.platform !=  'darwin':
    raise Exception('The pyautogui_osx module should only be loaded on an OS X system.')



""" Taken from events.h
/System/Library/Frameworks/Carbon.framework/Versions/A/Frameworks/HIToolbox.framework/Versions/A/Headers/Events.h

The *KB dictionaries in pyautogui map a string that can be passed to keyDown(),
keyUp(), or press() into the code used for the OS-specific keyboard function.

They should always be lowercase, and the same keys should be used across all OSes."""
keyboardMapping = dict([(key, None) for key in pyautogui.KEY_NAMES])
keyboardMapping.update({
    'a': 0x00, # kVK_ANSI_A
    's': 0x01, # kVK_ANSI_S
    'd': 0x02, # kVK_ANSI_D
    'f': 0x03, # kVK_ANSI_F
    'h': 0x04, # kVK_ANSI_H
    'g': 0x05, # kVK_ANSI_G
    'z': 0x06, # kVK_ANSI_Z
    'x': 0x07, # kVK_ANSI_X
    'c': 0x08, # kVK_ANSI_C
    'v': 0x09, # kVK_ANSI_V
    'b': 0x0b, # kVK_ANSI_B
    'q': 0x0c, # kVK_ANSI_Q
    'w': 0x0d, # kVK_ANSI_W
    'e': 0x0e, # kVK_ANSI_E
    'r': 0x0f, # kVK_ANSI_R
    'y': 0x10, # kVK_ANSI_Y
    't': 0x11, # kVK_ANSI_T
    '1': 0x12, # kVK_ANSI_1
    '!': 0x12, # kVK_ANSI_1
    '2': 0x13, # kVK_ANSI_2
    '@': 0x13, # kVK_ANSI_2
    '3': 0x14, # kVK_ANSI_3
    '#': 0x14, # kVK_ANSI_3
    '4': 0x15, # kVK_ANSI_4
    '$': 0x15, # kVK_ANSI_4
    '6': 0x16, # kVK_ANSI_6
    '^': 0x16, # kVK_ANSI_6
    '5': 0x17, # kVK_ANSI_5
    '%': 0x17, # kVK_ANSI_5
    '=': 0x18, # kVK_ANSI_Equal
    '+': 0x18, # kVK_ANSI_Equal
    '9': 0x19, # kVK_ANSI_9
    '(': 0x19, # kVK_ANSI_9
    '7': 0x1a, # kVK_ANSI_7
    '&': 0x1a, # kVK_ANSI_7
    '-': 0x1b, # kVK_ANSI_Minus
    '_': 0x1b, # kVK_ANSI_Minus
    '8': 0x1c, # kVK_ANSI_8
    '*': 0x1c, # kVK_ANSI_8
    '0': 0x1d, # kVK_ANSI_0
    ')': 0x1d, # kVK_ANSI_0
    ']': 0x1e, # kVK_ANSI_RightBracket
    '}': 0x1e, # kVK_ANSI_RightBracket
    'o': 0x1f, # kVK_ANSI_O
    'u': 0x20, # kVK_ANSI_U
    '[': 0x21, # kVK_ANSI_LeftBracket
    '{': 0x21, # kVK_ANSI_LeftBracket
    'i': 0x22, # kVK_ANSI_I
    'p': 0x23, # kVK_ANSI_P
    'l': 0x25, # kVK_ANSI_L
    'j': 0x26, # kVK_ANSI_J
    "'": 0x27, # kVK_ANSI_Quote
    '"': 0x27, # kVK_ANSI_Quote
    'k': 0x28, # kVK_ANSI_K
    ';': 0x29, # kVK_ANSI_Semicolon
    ':': 0x29, # kVK_ANSI_Semicolon
    '\\': 0x2a, # kVK_ANSI_Backslash
    '|': 0x2a, # kVK_ANSI_Backslash
    ',': 0x2b, # kVK_ANSI_Comma
    '<': 0x2b, # kVK_ANSI_Comma
    '/': 0x2c, # kVK_ANSI_Slash
    '?': 0x2c, # kVK_ANSI_Slash
    'n': 0x2d, # kVK_ANSI_N
    'm': 0x2e, # kVK_ANSI_M
    '.': 0x2f, # kVK_ANSI_Period
    '>': 0x2f, # kVK_ANSI_Period
    '`': 0x32, # kVK_ANSI_Grave
    '~': 0x32, # kVK_ANSI_Grave
    ' ': 0x31, # kVK_Space
    'space': 0x31,
    '\r': 0x24, # kVK_Return
    '\n': 0x24, # kVK_Return
    'enter': 0x24, # kVK_Return
    'return': 0x24, # kVK_Return
    '\t': 0x30, # kVK_Tab
    'tab': 0x30, # kVK_Tab
    'backspace': 0x33, # kVK_Delete, which is "Backspace" on OS X.
    '\b': 0x33, # kVK_Delete, which is "Backspace" on OS X.
    'esc': 0x35, # kVK_Escape
    'escape': 0x35, # kVK_Escape
    'command': 0x37, # kVK_Command
    'shift': 0x38, # kVK_Shift
    'shiftleft': 0x38, # kVK_Shift
    'capslock': 0x39, # kVK_CapsLock
    'option': 0x3a, # kVK_Option
    'optionleft': 0x3a, # kVK_Option
    'alt': 0x3a, # kVK_Option
    'altleft': 0x3a, # kVK_Option
    'ctrl': 0x3b, # kVK_Control
    'ctrlleft': 0x3b, # kVK_Control
    'shiftright': 0x3c, # kVK_RightShift
    'optionright': 0x3d, # kVK_RightOption
    'ctrlright': 0x3e, # kVK_RightControl
    'fn': 0x3f, # kVK_Function
    'f17': 0x40, # kVK_F17
    'volumeup': 0x48, # kVK_VolumeUp
    'volumedown': 0x49, # kVK_VolumeDown
    'volumemute': 0x4a, # kVK_Mute
    'f18': 0x4f, # kVK_F18
    'f19': 0x50, # kVK_F19
    'f20': 0x5a, # kVK_F20
    'f5': 0x60, # kVK_F5
    'f6': 0x61, # kVK_F6
    'f7': 0x62, # kVK_F7
    'f3': 0x63, # kVK_F3
    'f8': 0x64, # kVK_F8
    'f9': 0x65, # kVK_F9
    'f11': 0x67, # kVK_F11
    'f13': 0x69, # kVK_F13
    'f16': 0x6a, # kVK_F16
    'f14': 0x6b, # kVK_F14
    'f10': 0x6d, # kVK_F10
    'f12': 0x6f, # kVK_F12
    'f15': 0x71, # kVK_F15
    'help': 0x72, # kVK_Help
    'home': 0x73, # kVK_Home
    'pageup': 0x74, # kVK_PageUp
    'pgup': 0x74, # kVK_PageUp
    'del': 0x75, # kVK_ForwardDelete
    'delete': 0x75, # kVK_ForwardDelete
    'f4': 0x76, # kVK_F4
    'end': 0x77, # kVK_End
    'f2': 0x78, # kVK_F2
    'pagedown': 0x79, # kVK_PageDown
    'pgdn': 0x79, # kVK_PageDown
    'f1': 0x7a, # kVK_F1
    'left': 0x7b, # kVK_LeftArrow
    'right': 0x7c, # kVK_RightArrow
    'down': 0x7d, # kVK_DownArrow
    'up': 0x7e, # kVK_UpArrow
    'yen': 0x5d, # kVK_JIS_Yen
    #'underscore' : 0x5e, # kVK_JIS_Underscore (only applies to Japanese keyboards)
    #'comma': 0x5f, # kVK_JIS_KeypadComma (only applies to Japanese keyboards)
    'eisu': 0x66, # kVK_JIS_Eisu
    'kana': 0x68, # kVK_JIS_Kana
})

"""
# TODO - additional key codes to add
  kVK_ANSI_KeypadDecimal        = 0x41,
  kVK_ANSI_KeypadMultiply       = 0x43,
  kVK_ANSI_KeypadPlus           = 0x45,
  kVK_ANSI_KeypadClear          = 0x47,
  kVK_ANSI_KeypadDivide         = 0x4B,
  kVK_ANSI_KeypadEnter          = 0x4C,
  kVK_ANSI_KeypadMinus          = 0x4E,
  kVK_ANSI_KeypadEquals         = 0x51,
  kVK_ANSI_Keypad0              = 0x52,
  kVK_ANSI_Keypad1              = 0x53,
  kVK_ANSI_Keypad2              = 0x54,
  kVK_ANSI_Keypad3              = 0x55,
  kVK_ANSI_Keypad4              = 0x56,
  kVK_ANSI_Keypad5              = 0x57,
  kVK_ANSI_Keypad6              = 0x58,
  kVK_ANSI_Keypad7              = 0x59,
  kVK_ANSI_Keypad8              = 0x5B,
  kVK_ANSI_Keypad9              = 0x5C,
"""

# add mappings for uppercase letters
for c in 'abcdefghijklmnopqrstuvwxyz':
    keyboardMapping[c.upper()] = keyboardMapping[c]

# Taken from ev_keymap.h
# http://www.opensource.apple.com/source/IOHIDFamily/IOHIDFamily-86.1/IOHIDSystem/IOKit/hidsystem/ev_keymap.h
special_key_translate_table = {
    'KEYTYPE_SOUND_UP': 0,
    'KEYTYPE_SOUND_DOWN': 1,
    'KEYTYPE_BRIGHTNESS_UP': 2,
    'KEYTYPE_BRIGHTNESS_DOWN': 3,
    'KEYTYPE_CAPS_LOCK': 4,
    'KEYTYPE_HELP': 5,
    'POWER_KEY': 6,
    'KEYTYPE_MUTE': 7,
    'UP_ARROW_KEY': 8,
    'DOWN_ARROW_KEY': 9,
    'KEYTYPE_NUM_LOCK': 10,
    'KEYTYPE_CONTRAST_UP': 11,
    'KEYTYPE_CONTRAST_DOWN': 12,
    'KEYTYPE_LAUNCH_PANEL': 13,
    'KEYTYPE_EJECT': 14,
    'KEYTYPE_VIDMIRROR': 15,
    'KEYTYPE_PLAY': 16,
    'KEYTYPE_NEXT': 17,
    'KEYTYPE_PREVIOUS': 18,
    'KEYTYPE_FAST': 19,
    'KEYTYPE_REWIND': 20,
    'KEYTYPE_ILLUMINATION_UP': 21,
    'KEYTYPE_ILLUMINATION_DOWN': 22,
    'KEYTYPE_ILLUMINATION_TOGGLE': 23
}

def _keyDown(key):
    if key not in keyboardMapping or keyboardMapping[key] is None:
        return

    if key in special_key_translate_table:
        _specialKeyEvent(key, 'down')
    else:
        _normalKeyEvent(key, 'down')

def _keyUp(key):
    if key not in keyboardMapping or keyboardMapping[key] is None:
        return

    if key in special_key_translate_table:
        _specialKeyEvent(key, 'up')
    else:
        _normalKeyEvent(key, 'up')


def _normalKeyEvent(key, upDown):
    assert upDown in ('up', 'down'), "upDown argument must be 'up' or 'down'"

    try:
        if pyautogui.isShiftCharacter(key):
            key_code = keyboardMapping[key.lower()]

            event = Quartz.CGEventCreateKeyboardEvent(None,
                        keyboardMapping['shift'], upDown == 'down')
            Quartz.CGEventPost(Quartz.kCGHIDEventTap, event)
            # Tiny sleep to let OS X catch up on us pressing shift
            time.sleep(pyautogui.DARWIN_CATCH_UP_TIME)

        else:
            key_code = keyboardMapping[key]

        event = Quartz.CGEventCreateKeyboardEvent(None, key_code, upDown == 'down')
        Quartz.CGEventPost(Quartz.kCGHIDEventTap, event)
        time.sleep(pyautogui.DARWIN_CATCH_UP_TIME)

    # TODO - wait, is the shift key's keyup not done?
    # TODO - get rid of this try-except.

    except KeyError:
        raise RuntimeError("Key %s not implemented." % (key))

def _specialKeyEvent(key, upDown):
    """ Helper method for special keys.

    Source: http://stackoverflow.com/questions/11045814/emulate-media-key-press-on-mac
    """
    assert upDown in ('up', 'down'), "upDown argument must be 'up' or 'down'"

    key_code = special_key_translate_table[key]

    ev = AppKit.NSEvent.otherEventWithType_location_modifierFlags_timestamp_windowNumber_context_subtype_data1_data2_(
            Quartz.NSSystemDefined, # type
            (0,0), # location
            0xa00 if upDown == 'down' else 0xb00, # flags
            0, # timestamp
            0, # window
            0, # ctx
            8, # subtype
            (key_code << 16) | ((0xa if upDown == 'down' else 0xb) << 8), # data1
            -1 # data2
        )

    Quartz.CGEventPost(0, ev.CGEvent())









def _position():
    loc = AppKit.NSEvent.mouseLocation()
    return int(loc.x), int(Quartz.CGDisplayPixelsHigh(0) - loc.y)


def _size():
    return Quartz.CGDisplayPixelsWide(Quartz.CGMainDisplayID()), Quartz.CGDisplayPixelsHigh(Quartz.CGMainDisplayID())



def _scroll(clicks, x=None, y=None):
    _vscroll(clicks, x, y)


"""
According to https://developer.apple.com/library/mac/documentation/Carbon/Reference/QuartzEventServicesRef/Reference/reference.html#//apple_ref/c/func/Quartz.CGEventCreateScrollWheelEvent
"Scrolling movement is generally represented by small signed integer values, typically in a range from -10 to +10. Large values may have unexpected results, depending on the application that processes the event."
The scrolling functions will create multiple events that scroll 10 each, and then scroll the remainder.
"""

def _vscroll(clicks, x=None, y=None):
    _moveTo(x, y)
    clicks = int(clicks)
    for _ in range(abs(clicks) // 10):
        scrollWheelEvent = Quartz.CGEventCreateScrollWheelEvent(
            None, # no source
            Quartz.kCGScrollEventUnitLine, # units
            1, # wheelCount (number of dimensions)
            10 if clicks >= 0 else -10) # vertical movement
        Quartz.CGEventPost(Quartz.kCGHIDEventTap, scrollWheelEvent)

    scrollWheelEvent = Quartz.CGEventCreateScrollWheelEvent(
        None, # no source
        Quartz.kCGScrollEventUnitLine, # units
        1, # wheelCount (number of dimensions)
        clicks % 10 if clicks >= 0 else -1 * (-clicks % 10)) # vertical movement
    Quartz.CGEventPost(Quartz.kCGHIDEventTap, scrollWheelEvent)


def _hscroll(clicks, x=None, y=None):
    _moveTo(x, y)
    clicks = int(clicks)
    for _ in range(abs(clicks) // 10):
        scrollWheelEvent = Quartz.CGEventCreateScrollWheelEvent(
            None, # no source
            Quartz.kCGScrollEventUnitLine, # units
            2, # wheelCount (number of dimensions)
            0, # vertical movement
            10 if clicks >= 0 else -10) # horizontal movement
        Quartz.CGEventPost(Quartz.kCGHIDEventTap, scrollWheelEvent)

    scrollWheelEvent = Quartz.CGEventCreateScrollWheelEvent(
        None, # no source
        Quartz.kCGScrollEventUnitLine, # units
        2, # wheelCount (number of dimensions)
        0, # vertical movement
        (clicks % 10) if clicks >= 0 else (-1 * clicks % 10)) # horizontal movement
    Quartz.CGEventPost(Quartz.kCGHIDEventTap, scrollWheelEvent)


def _mouseDown(x, y, button):
    if button == LEFT:
        _sendMouseEvent(Quartz.kCGEventLeftMouseDown, x, y, Quartz.kCGMouseButtonLeft)
    elif button == MIDDLE:
        _sendMouseEvent(Quartz.kCGEventOtherMouseDown, x, y, Quartz.kCGMouseButtonCenter)
    elif button == RIGHT:
        _sendMouseEvent(Quartz.kCGEventRightMouseDown, x, y, Quartz.kCGMouseButtonRight)
    else:
        assert False, "button argument not in ('left', 'middle', 'right')"


def _mouseUp(x, y, button):
    if button == LEFT:
        _sendMouseEvent(Quartz.kCGEventLeftMouseUp, x, y, Quartz.kCGMouseButtonLeft)
    elif button == MIDDLE:
        _sendMouseEvent(Quartz.kCGEventOtherMouseUp, x, y, Quartz.kCGMouseButtonCenter)
    elif button == RIGHT:
        _sendMouseEvent(Quartz.kCGEventRightMouseUp, x, y, Quartz.kCGMouseButtonRight)
    else:
        assert False, "button argument not in ('left', 'middle', 'right')"


def _click(x, y, button):
    if button == LEFT:
        _sendMouseEvent(Quartz.kCGEventLeftMouseDown, x, y, Quartz.kCGMouseButtonLeft)
        _sendMouseEvent(Quartz.kCGEventLeftMouseUp, x, y, Quartz.kCGMouseButtonLeft)
    elif button == MIDDLE:
        _sendMouseEvent(Quartz.kCGEventOtherMouseDown, x, y, Quartz.kCGMouseButtonCenter)
        _sendMouseEvent(Quartz.kCGEventOtherMouseUp, x, y, Quartz.kCGMouseButtonCenter)
    elif button == RIGHT:
        _sendMouseEvent(Quartz.kCGEventRightMouseDown, x, y, Quartz.kCGMouseButtonRight)
        _sendMouseEvent(Quartz.kCGEventRightMouseUp, x, y, Quartz.kCGMouseButtonRight)
    else:
        assert False, "button argument not in ('left', 'middle', 'right')"

def _multiClick(x, y, button, num, interval=0.0):
    btn    = None
    down   = None
    up     = None

    if button == LEFT:
        btn  = Quartz.kCGMouseButtonLeft
        down = Quartz.kCGEventLeftMouseDown
        up   = Quartz.kCGEventLeftMouseUp
    elif button == MIDDLE:
        btn  = Quartz.kCGMouseButtonCenter
        down = Quartz.kCGEventOtherMouseDown
        up   = Quartz.kCGEventOtherMouseUp
    elif button == RIGHT:
        btn  = Quartz.kCGMouseButtonRight
        down = Quartz.kCGEventRightMouseDown
        up   = Quartz.kCGEventRightMouseUp
    else:
        assert False, "button argument not in ('left', 'middle', 'right')"
        return

    for i in range(num):
        _click(x, y, button)
        time.sleep(interval)


def _sendMouseEvent(ev, x, y, button):
    mouseEvent = Quartz.CGEventCreateMouseEvent(None, ev, (x, y), button)
    Quartz.CGEventPost(Quartz.kCGHIDEventTap, mouseEvent)


def _dragTo(x, y, button):
    if button == LEFT:
        _sendMouseEvent(Quartz.kCGEventLeftMouseDragged , x, y, Quartz.kCGMouseButtonLeft)
    elif button == MIDDLE:
        _sendMouseEvent(Quartz.kCGEventOtherMouseDragged , x, y, Quartz.kCGMouseButtonCenter)
    elif button == RIGHT:
        _sendMouseEvent(Quartz.kCGEventRightMouseDragged , x, y, Quartz.kCGMouseButtonRight)
    else:
        assert False, "button argument not in ('left', 'middle', 'right')"
    time.sleep(pyautogui.DARWIN_CATCH_UP_TIME) # needed to allow OS time to catch up.

def _moveTo(x, y):
    _sendMouseEvent(Quartz.kCGEventMouseMoved, x, y, 0)
    time.sleep(pyautogui.DARWIN_CATCH_UP_TIME) # needed to allow OS time to catch up.



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import os
import sys
import subprocess
import cairo
from . import predicate
import errno
import shlex

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

from gi.repository import Gtk, GLib
from .config import config
from time import sleep
from .logging import debugLogger as logger
from .logging import TimeStamp

"""
Various utilities
"""

__author__ = """Ed Rousseau <rousseau@redhat.com>,
Zack Cerza <zcerza@redhat.com,
David Malcolm <dmalcolm@redhat.com>
"""


def screenshot(file='screenshot.png', timeStamp=True):
    """
    This function wraps the ImageMagick import command to take a screenshot.

    The file argument may be specified as 'foo', 'foo.png', or using any other
    extension that ImageMagick supports. PNG is the default.

    By default, screenshot filenames are in the format of foo_YYYYMMDD-hhmmss.png .
    The timeStamp argument may be set to False to name the file foo.png.
    """
    if not isinstance(timeStamp, bool):
        raise TypeError("timeStampt must be True or False")
    # config is supposed to create this for us. If it's not there, bail.
    assert os.path.isdir(config.scratchDir)

    baseName = ''.join(file.split('.')[0:-1])
    fileExt = file.split('.')[-1].lower()
    if not baseName:
        baseName = file
        fileExt = 'png'

    if timeStamp:
        ts = TimeStamp()
        newFile = ts.fileStamp(baseName) + '.' + fileExt
        path = config.scratchDir + newFile
    else:
        newFile = baseName + '.' + fileExt
        path = config.scratchDir + newFile

    from gi.repository import Gdk
    from gi.repository import GdkPixbuf
    rootWindow = Gdk.get_default_root_window()
    geometry = rootWindow.get_geometry()
    pixbuf = GdkPixbuf.Pixbuf(colorspace=GdkPixbuf.Colorspace.RGB,
                              has_alpha=False,
                              bits_per_sample=8,
                              width=geometry[2],
                              height=geometry[3])

    pixbuf = Gdk.pixbuf_get_from_window(rootWindow, 0, 0, geometry[2], geometry[3])
    # GdkPixbuf.Pixbuf.save() needs 'jpeg' and not 'jpg'
    if fileExt == 'jpg':
        fileExt = 'jpeg'
    try:
        pixbuf.savev(path, fileExt, [], [])
    except GLib.GError:
        raise ValueError("Failed to save screenshot in %s format" % fileExt)
    assert os.path.exists(path)
    logger.log("Screenshot taken: " + path)
    return path


def run(string, timeout=config.runTimeout, interval=config.runInterval, desktop=None, dumb=False, appName=''):
    """
    Runs an application. [For simple command execution such as 'rm *', use os.popen() or os.system()]
    If dumb is omitted or is False, polls at interval seconds until the application is finished starting, or until
    timeout is reached.
    If dumb is True, returns when timeout is reached.
    """
    if not desktop:
        from dogtail.tree import root as desktop
    args = shlex.split(string)
    os.environ['GTK_MODULES'] = 'gail:atk-bridge'
    pid = subprocess.Popen(args, env=os.environ).pid

    if not appName:
        appName = args[0]

    if dumb:
        # We're starting a non-AT-SPI-aware application. Disable startup
        # detection.
        doDelay(timeout)
    else:
        # Startup detection code
        # The timing here is not totally precise, but it's good enough for now.
        time = 0
        while time < timeout:
            time = time + interval
            try:
                for child in desktop.children[::-1]:
                    if child.name == appName:
                        for grandchild in child.children:
                            if grandchild.roleName == 'frame':
                                from dogtail.procedural import focus
                                focus.application.node = child
                                doDelay(interval)
                                return pid
            except AttributeError:  # pragma: no cover
                pass
            doDelay(interval)
    return pid


def doDelay(delay=None):
    """
    Utility function to insert a delay (with logging and a configurable
    default delay)
    """
    if delay is None:
        delay = config.defaultDelay
    if config.debugSleep:
        logger.log("sleeping for %f" % delay)
    sleep(delay)


class Highlight (Gtk.Window):  # pragma: no cover

    def __init__(self, x, y, w, h):  # pragma: no cover
        super(Highlight, self).__init__()
        self.set_decorated(False)
        self.set_has_resize_grip(False)
        self.set_default_size(w, h)
        self.screen = self.get_screen()
        self.visual = self.screen.get_rgba_visual()
        if self.visual is not None and self.screen.is_composited():
            self.set_visual(self.visual)
        self.set_app_paintable(True)
        self.connect("draw", self.area_draw)
        self.show_all()
        self.move(x, y)

    def area_draw(self, widget, cr):  # pragma: no cover
        cr.set_source_rgba(.0, .0, .0, 0.0)
        cr.set_operator(cairo.OPERATOR_SOURCE)
        cr.paint()
        cr.set_operator(cairo.OPERATOR_OVER)
        cr.set_source_rgb(0.9, 0.1, 0.1)
        cr.set_line_width(6)
        cr.rectangle(0, 0, self.get_size()[0], self.get_size()[1])
        cr.stroke()


class Blinker(object):  # pragma: no cover
    INTERVAL_MS = 1000
    main_loop = GLib.MainLoop()

    def __init__(self, x, y, w, h):  # pragma: no cover
        self.highlight_window = Highlight(x, y, w, h)
        if self.highlight_window.screen.is_composited() is not False:
            self.timeout_handler_id = GLib.timeout_add(
                Blinker.INTERVAL_MS, self.destroyHighlight)
            self.main_loop.run()
        else:
            self.highlight_window.destroy()

    def destroyHighlight(self):  # pragma: no cover
        self.highlight_window.destroy()
        self.main_loop.quit()
        return False


class Lock(object):
    """
    A known sytem-wide mutex implementation that uses atomicity of the mkdir operation in UNIX-like
    systems. This should be used mainly to provide mutual exclution in handling possible collisions
    among multiple script instances. You can choose to make randomized single-script wise locks
    or a more general locks if you do not choose to randomize the lockdir name. Set unLockOnExit
    to True to enable automatic unlock when scipt process exits to avoid having to unlock manually.
    """

    def __init__(self, location='/tmp', lockname='dogtail_lockdir_', randomize=True, unlockOnExit=False):
        """
        You can change the default lockdir location or name. Setting randomize to
        False will result in no random string being appened to the lockdir name.
        """
        self.lockdir = os.path.join(os.path.normpath(location), lockname)
        if randomize:
            self.lockdir = "%s%s" % (self.lockdir, self.__getPostfix())
        self.unlockOnExit = unlockOnExit

    def __exit_unlock(self):
        if os.path.exists(self.lockdir):
            try:
                os.rmdir(self.lockdir)
            except OSError:
                pass  # already deleted (by .unlock()), we're exiting, it's fine

    def lock(self):
        """
        Creates a lockdir based on the settings on Lock() instance creation.
        Raises OSError exception of the lock is already present. Should be
        atomic on POSIX compliant systems.
        """
        locked_msg = 'Dogtail lock: Already locked with the same lock'
        if not os.path.exists(self.lockdir):
            try:
                os.mkdir(self.lockdir)
            except OSError as e:
                if e.errno == errno.EEXIST and os.path.isdir(self.lockdir):
                    raise OSError(locked_msg)
            if os.path.exists(self.lockdir):
                if self.unlockOnExit:
                    import atexit
                    atexit.register(self.__exit_unlock)
                return self.lockdir
        else:
            raise OSError(locked_msg)

    def unlock(self):
        """
        Removes a lock. Will raise OSError exception if the lock was not present.
        Should be atomic on POSIX compliant systems.
        """
        #if self.unlockOnExit:
        #    raise Exception('Cannot unlock with unlockOnExit set to True!')
        if os.path.exists(self.lockdir):
            try:
                os.rmdir(self.lockdir)
            except OSError as e:
                if e.erron == errno.EEXIST:
                    raise OSError('Dogtail unlock: lockdir removed elsewhere!')
        else:
            raise OSError('Dogtail unlock: not locked')

    def locked(self):
        return os.path.exists(self.lockdir)

    def __getPostfix(self):
        import random
        import string
        return ''.join(random.choice(string.ascii_letters + string.digits) for x in range(5))


a11yDConfKey = 'org.gnome.desktop.interface'


def isA11yEnabled():
    """
    Checks if accessibility is enabled via DConf.
    """
    from gi.repository.Gio import Settings
    try:
        InterfaceSettings = Settings(schema_id=a11yDConfKey)
    except TypeError: # if we have older glib that has deprecated param name
        InterfaceSettings = Settings(schema=a11yDConfKey)
    dconfEnabled = InterfaceSettings.get_boolean('toolkit-accessibility')
    if os.environ.get('GTK_MODULES', '').find('gail:atk-bridge') == -1:
        envEnabled = False
    else:
        envEnabled = True  # pragma: no cover
    return (dconfEnabled or envEnabled)


def bailBecauseA11yIsDisabled():
    if sys.argv[0].endswith("pydoc"):
        return
    try:
        with open("/proc/%s/cmdline" % os.getpid(), 'r') as f:
            content = f.read()
        if content.find('epydoc') != -1:
            return  # pragma: no cover
        if content.find('sphinx') != -1:
            return  # pragma: no cover
    except:   # pragma: no cover
        pass  # pragma: no cover
    logger.log("Dogtail requires that Assistive Technology support be enabled."
               "\nYou can enable accessibility with sniff or by running:\n"
               "'gsettings set org.gnome.desktop.interface toolkit-accessibility true'\nAborting...")
    sys.exit(1)


def enableA11y(enable=True):
    """
    Enables accessibility via DConf.
    """
    from gi.repository.Gio import Settings
    InterfaceSettings = Settings(schema_id=a11yDConfKey)
    InterfaceSettings.set_boolean('toolkit-accessibility', enable)


def checkForA11y():
    """
    Checks if accessibility is enabled, and halts execution if it is not.
    """
    if not isA11yEnabled():  # pragma: no cover
        bailBecauseA11yIsDisabled()


def checkForA11yInteractively():  # pragma: no cover
    """
    Checks if accessibility is enabled, and presents a dialog prompting the
    user if it should be enabled if it is not already, then halts execution.
    """
    if isA11yEnabled():
        return
    dialog = Gtk.Dialog('Enable Assistive Technology Support?',
                        None,
                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                        (Gtk.STOCK_QUIT, Gtk.ResponseType.CLOSE,
                         "_Enable", Gtk.ResponseType.ACCEPT))
    question = """Dogtail requires that Assistive Technology Support be enabled for it to function. Would you like to
enable Assistive Technology support now?

Note that you will have to log out for the change to fully take effect.
    """.strip()
    dialog.set_default_response(Gtk.ResponseType.ACCEPT)
    questionLabel = Gtk.Label(label=question)
    questionLabel.set_line_wrap(True)
    dialog.vbox.pack_start(questionLabel, True, True, 0)
    dialog.show_all()
    result = dialog.run()
    if result == Gtk.ResponseType.ACCEPT:
        logger.log("Enabling accessibility...")
        enableA11y()
    elif result == Gtk.ResponseType.CLOSE:
        bailBecauseA11yIsDisabled()
    dialog.destroy()


class GnomeShell(object):  # pragma: no cover

    """
    Utility class to help working with certain atributes of gnome-shell.
    Currently that means handling the Application menu available for apps
    on the top gnome-shell panel. Searching for the menu and its items is
    somewhat tricky due to fuzzy a11y tree of gnome-shell, mainly since the
    actual menu is not present as child to the menu-spawning button. Also,
    the menus get constructed/destroyed on the fly with application focus
    changes. Thus current application name as displayed plus a reference
    known menu item (with 'Quit' as default) are required by these methods.
    """

    def __init__(self, classic_mode=False):
        from dogtail.tree import root
        self.shell = root.application('gnome-shell')

    def getApplicationMenuList(self, search_by_item='Quit'):
        """
        Returns list of all menu item nodes. Searches for the menu by a reference item.
        Provide a different item name, if the 'Quit' is not present - but beware picking one
        present elsewhere, like 'Lock' or 'Power Off' present under the user menu.
        """
        matches = self.shell.findChildren(
            predicate.GenericPredicate(name=search_by_item, roleName='label'))
        for match in matches:
            ancestor = match.parent.parent.parent
            if ancestor.roleName == 'panel':
                return ancestor.findChildren(predicate.GenericPredicate(roleName='label'))
        from dogtail.tree import SearchError
        raise SearchError(
            "Could not find the Application menu based on '%s' item. Please provide an existing reference item"
            % search_by_item)

    def getApplicationMenuButton(self, app_name):
        """
        Returns the application menu 'button' node as present on the gnome-shell top panel.
        """
        try:
            return self.shell[0][0][3].child(app_name, roleName='label')
        except:
            from dogtail.tree import SearchError
            raise SearchError(
                "Application menu button of %s could not be found within gnome-shell!" % app_name)

    def getApplicationMenuItem(self, item, search_by_item='Quit'):
        """
        Returns a particilar menu item node. Uses a different 'Quit' or custom item name for reference, but also
        attempts to use the given item if the general reference fails.
        """
        try:
            menu_items = self.getApplicationMenuList(search_by_item)
        except:
            menu_items = self.getApplicationMenuList(item)
        if any(ord(x) > 127 for x in item):
            item = item.encode('utf-8')
        for node in menu_items:
            if node.name == item:
                return node
        raise Exception(
            'Could not find the item, did application focus change?')

    def clickApplicationMenuItem(self, app_name, item, search_by_item='Quit'):
        """
        Executes the given menu item through opening the menu first followed
        by a click at the particular item. The menu search reference 'Quit'
        may be customized. Also attempts to use the given item for reference
        if search fails with the default/custom one.
        """

        # a problem with this version of GS misreporting the positions
        from subprocess import check_output
        if 'Fedora release 25' in check_output('cat /etc/redhat-release', shell=True):
            xoffset = -130
            from dogtail.rawinput import click
            nd = self.getApplicationMenuButton(app_name)
            x = nd.position[0] + nd.size[0] / 2 + xoffset
            y = nd.position[1] + nd.size[1] / 2
            click(x, y)
            nd = self.getApplicationMenuItem(item, search_by_item)
            x = nd.position[0] + nd.size[0] / 2 + xoffset
            y = nd.position[1] + nd.size[1] / 2
            click(x, y)
        else:
            self.getApplicationMenuButton(app_name).click()
            self.getApplicationMenuItem(item, search_by_item).click()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
from time import sleep
from setting import conf


def git_config():
    os.system("git config --global http.lowSpeedLimit 0")
    os.system("git config --global http.lowSpeedTime 999999")


def sslclone(
    git_url: str = None,
    git_user: str = None,
    git_password: str = None,
    branch: str = None,
    path_to: str = None,
    depth: [str, int] = None,
    **kwargs,
):
    branch = branch or conf.BRANCH
    depth = depth or conf.DEPTH
    if path_to is None:
        path_to = "apps"
    git_config()
    clone_cmd = (
        f"cd {conf.ROOT_DIR}/src/utils && "
        f"bash sslclone.sh {conf.ROOT_DIR}/{path_to} "
        f"{git_url or conf.GIT_URL} "
        f"{git_user or conf.GTI_USER} {git_password or conf.GIT_PASSWORD} "
        f"{branch or ''} {depth or ''}"
    )
    print(clone_cmd)
    os.system(clone_cmd)
    # relax
    sleep(2)


def clone(
        git_url: str = None,
        branch: str = "",
        path_to: str = None,
        depth: [str, int] = "",
        **kwargs
):
    branch = branch or conf.BRANCH
    depth = depth or conf.DEPTH
    if path_to is None:
        path_to = "apps"
    git_config()
    clone_cmd = (
        f"cd {conf.ROOT_DIR}/{path_to} && git clone "
        f"{git_url or conf.GIT_URL} "
        f"{f'-b {branch}' if branch else ''} {f'--depth {depth}' if depth else ''}"
    )
    print(clone_cmd)
    os.system(clone_cmd)
    # relax
    sleep(2)



# -*- coding: utf-8 -*-

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114,C0103,R0903
import os
from tkinter import Tk
from functools import partial

from setting.globalconfig import GlobalConfig
from src.rtk._base import collect_result


def wf(f, t):
    f.write(t.encode("unicode_escape").decode() + "\n")


class AllureReportExtend:
    @staticmethod
    def environment_info(session, execute):
        try:
            allure_path = session.config.option.allure_report_dir
        except TypeError:
            return
        if not allure_path:
            return
        allure_fspath_path = os.path.join(
            session.config.invocation_dir, allure_path, "environment.properties"
        )
        with open(allure_fspath_path, "w+", encoding="utf-8") as _f:
            w = partial(wf, _f)

            py_case_info = ""
            if execute:
                total, failed, passed, skiped, _ = collect_result(execute)
                py_case_info = f"{total}/{passed}/{failed}/{skiped}"

            w(f"Py用例维度(总数/通过/失败/跳过)={py_case_info}")
            w(f"网络地址={GlobalConfig.USERNAME}@{GlobalConfig.HOST_IP}")
            w(f"工作目录={GlobalConfig.ROOT_DIR}")
            w(f"镜像版本={GlobalConfig.VERSION}")

            screen = Tk()
            w(f"分辨率={screen.winfo_screenwidth()}x{screen.winfo_screenheight()}")
            w(f"显示协议={GlobalConfig.DISPLAY_SERVER.title()}")

            try:
                cpu_info = os.popen('cat /proc/cpuinfo | grep "model name"').readlines()
                if cpu_info:
                    w(f"CPU信息={cpu_info[0]}")
                mem_info = os.popen('cat /proc/meminfo | grep MemTotal').readlines()
                if mem_info:
                    w(f"内存信息={mem_info[0]}")
            except IndexError:
                ...

            os_info = os.popen("uname -a").read()
            w(f"内核信息={os_info}")
